(***************************************************************)
(* PACKAGE: EXPORTPACK 1.0                                     *)
(***************************************************************)
(* Autor: Ismael Fernndez Bustos (ismafb@terra.es)            *)
(* Notas: Exportadores de informes de quickreport y fastreport *)
(*        a formatos grficos (JPG, BMP, WMF, EMF y PDF).      *)
(*        La exportacin a PDF utiliza el componente TPrintPDF *)
(*        de K. Nishita (thanks) y ZLIB (thanks).              *)
(*                                                             *)
(* Fecha: 09/04/2002                                           *)
(***************************************************************)
unit ExportPack;

{$DEFINE QUICKREPORT}
{-$DEFINE FASTREPORT}
{.$DEFINE SPANISH}
{.DEFINE ENGLISH}
{$DEFINE FRENCH}

interface

uses
  WinProcs, Windows, Messages, SysUtils, Classes, QuickRpt, Graphics, JPEG, Forms,
  Dialogs, StdCtrls, ExtCtrls, QRPrntr{$IFDEF FASTREPORT}, FR_Class, FR_View{$ENDIF};

const
{$IFDEF SPANISH}
  Filtro = 'Archivo PDF (Alta Compresin)|*.PDF|Archivo PDF (Menor Compresin)|*.PDF|Archivos JPG|*.JPG|Archivos BMP|*.BMP|Archivos EMF|*.EMF|Archivos WMF|*.WMF';
  Error = 'Error exportando informe';
  Guardar = 'Guardar informe como...';
  Exportando = 'Exportando...';
  Pagina1De = 'Pgina... 1 de';
  Pagina = 'Pgina... ';
  De = ' de ';
{$ENDIF}

{$IFDEF ENGLISH}
  Filtro = 'PDF File (High Compression)|*.PDF|PDF File (Low Compression)|*.PDF|JPG Files|*.JPG|BMP Files|*.BMP|EMF Files|*.EMF|WMF Files|*.WMF';
  Error = 'Error exporting report';
  Guardar = 'Save report as...';
  Exportando = 'Exporting...';
  Pagina1De = 'Page... 1 of';
  Pagina = 'Page... ';
  De = ' of ';
{$ENDIF}

{$IFDEF FRENCH}
  Filtro = 'Fichier PDF (Haute Compression)|*.PDF|Fichier PDF (Basse Compression)|*.PDF|Fichier JPG|*.JPG|Fichier BMP|*.BMP|Fichier EMF|*.EMF|Fichier WMF|*.WMF';
  Error = 'Erreur d''exportation de l''impression';
  Guardar = 'Enregistrer Sous...';
  Exportando = 'Export en cours...';
  Pagina1De = 'Page... 1 de';
  Pagina = 'Page... ';
  De = ' de ';
{$ENDIF}

type
{$IFDEF QUICKREPORT}
  { TExportQR have ExportQRFromPreview because in one TQuickRep you
    only can view a report and if you make a personal preview you
    can't export }
  TExportQR = class(TComponent)
  private
    { Private declarations }
    FReport: TQuickRep;
    FPreview: TQRPreview;
    FTitle,
    FAuthor,
    FCreator,
    FKeywords,
    FSubject,
    FProducer: String;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Report: TQuickRep write FReport;
    property Preview: TQRPreview write FPreview;
    function ExportQR: Boolean; {Only Export Report}
    function ExportQRFromPreview: boolean;
    procedure ExportQRToStream(Stream: TStream; Compresion: Boolean = True);
    procedure ExportQRBMP(FileName: string; FromPreview: boolean = False);
    procedure ExportQRJPG(FileName: string; FromPreview: boolean = False);
    procedure ExportQREMF(FileName: string; FromPreview: boolean = False);
    procedure ExportQRWMF(FileName: string; FromPreview: boolean = False);
    procedure ExportQRPDF(FileName: string; Compresion: boolean; FromPreview: boolean = False);
  published
    { Published declarations }
    Property Title: String read FTitle write FTitle;
    Property Author: String read FAuthor write FAuthor;
    Property Creator: String read FCreator write FCreator;
    Property Keywords: String read FKeywords write FKeywords;
    Property Subject: String read FSubject write FSubject;
    Property Producer: String read FProducer write FProducer;
  end;
{$ENDIF}

{$IFDEF FASTREPORT}
  { TExportFR don't have ExportFRFromPreview method because
    in only one TfrReport you can view varios reports }
  TExportFR = class(TComponent)
  private
    { Private declarations }
    FReport: TFRReport;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Report: TFRReport write FReport;
    procedure ExportFR; {Only export report}
    procedure ExportFRBMP(FileName: string);
    procedure ExportFRJPG(FileName: string);
    procedure ExportFREMF(FileName: string);
    procedure ExportFRWMF(FileName: string);
    procedure ExportFRPDF(FileName: string; Compresion: boolean);
  published
    { Published declarations }
  end;
{$ENDIF}

procedure Register;

implementation

uses
  TnPDF, Prgrs, Printers;

procedure Register;
begin
{$IFDEF QUICKREPORT}
  RegisterComponents('QReport', [TExportQR]);
{$ENDIF}
{$IFDEF FASTREPORT}
  RegisterComponents('ExportPack', [TExportFR]);
{$ENDIF}
end;

// Funciones para cadenas

function PadL(Cad: string; Tam: integer; Car: string = '0'): string;
var
  i: integer;
  R: string;
begin
  R := '';
  for i := 1 to (Tam - length(Cad)) do
  begin
    R := R + Car;
  end;
  R := R + Cad;
  Result := R;
end;

function PadR(Cad: string; Tam: integer; Car: string = '0'): string;
var
  i: integer;
  R: string;
begin
  R := '';
  R := R + Cad;
  for i := 1 to (Tam - length(Cad)) do
  begin
    R := R + Car;
  end;
  Result := R;
end;

(*****************************)
(*        TEXPORTQR          *)
(*****************************)
{$IFDEF QUICKREPORT}

constructor TExportQR.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TExportQR.Destroy;
begin
  inherited Destroy;
end;

function TExportQR.ExportQR: Boolean;
var
  SD: TSaveDialog;
  Ruta: string;
  Fich: string;
  P: integer;
begin
  Result := False;
  if (not Assigned(FReport)) then Exit;
  { Intentamos abrir el dataset del report }
  if Assigned(FReport.DataSet) then
  begin
    try
      FReport.DataSet.Open;
    except
    end;
  end;
  FReport.Prepare; {Prepare report}
  try
    // Sacamos el cuadro de dilogo para guardar
    SD := TSaveDialog.Create(Self);
    SD.Title := Guardar;
    SD.Filter := Filtro;
    SD.FilterIndex := 0;
    if (SD.Execute) then
    begin
      Ruta := ExtractFilePath(SD.FileName);
      Fich := ExtractFileName(SD.FileName);
      P := Pos('.', Fich);
      if (P <> 0) then
        Fich := Copy(Fich, 1, P - 1);
      case SD.FilterIndex of
        1:
          begin
            ExportQRPDF(Ruta + Fich, True);
            Result := True;
          end;
        2:
          begin
            ExportQRPDF(Ruta + Fich, False);
            Result := True;
          end;
        3:
          begin
            ExportQRJPG(Ruta + Fich);
            Result := True;
          end;
        4:
          begin
            ExportQRBMP(Ruta + Fich);
            Result := True;
          end;
        5:
          begin
            ExportQREMF(Ruta + Fich);
            Result := True;
          end;
        6:
          begin
            ExportQRWMF(Ruta + Fich);
            Result := True;
          end;
      end;
    end;
  finally
    SD.Free;
    FReport.DataSet.Close;
    FReport.QRPrinter.Free;
    FReport.QRPrinter := nil;
  end;
end;

function TExportQR.ExportQRFromPreview: Boolean;
var
  SD: TSaveDialog;
  Ruta: string;
  Fich: string;
  P: integer;
begin
  Result := False;
  if (not Assigned(FPreview)) then Exit;
  try
    // Sacamos el cuadro de dilogo para guardar
    SD := TSaveDialog.Create(Self);
    SD.Title := Guardar;
    SD.Filter := Filtro;
    SD.FilterIndex := 0;
    if (SD.Execute) then
    begin
      Ruta := ExtractFilePath(SD.FileName);
      Fich := ExtractFileName(SD.FileName);
      P := Pos('.', Fich);
      if (P <> 0) then
        Fich := Copy(Fich, 1, P - 1);
      case SD.FilterIndex of
        1:
          begin
            ExportQRPDF(Ruta + Fich, True, True);
            Result := True;
          end;
        2:
          begin
            ExportQRPDF(Ruta + Fich, False, True);
            Result := True;
          end;
        3:
          begin
            ExportQRJPG(Ruta + Fich, True);
            Result := True;
          end;
        4:
          begin
            ExportQRBMP(Ruta + Fich, True);
            Result := True;
          end;
        5:
          begin
            ExportQREMF(Ruta + Fich, True);
            Result := True;
          end;
        6:
          begin
            ExportQRWMF(Ruta + Fich, True);
            Result := True;
          end;
      end;
    end;
  finally
    SD.Free;
  end;
end;

procedure TExportQR.ExportQRBMP(FileName: string; FromPreview: boolean = False);
var
  i: integer;
  MF: TMetaFile;
  B: TBitmap;
  Prg: TFPrgrs;
  QRPrinter: TQRPrinter;
begin
  if (not FromPreview) then
    QRPrinter := FReport.QRPrinter
  else
    QRPrinter := FPreview.QRPrinter;
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := QRPrinter.PageCount - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(QRPrinter.PageCount);
    Prg.Accion.Update;
    Prg.Show;
    for i := 1 to QRPrinter.PageCount do
    begin
      Prg.Accion.Caption := Pagina + IntToStr(i) + De + IntToStr(QRPrinter.PageCount);
      Prg.Accion.Update;
      B := TBitmap.Create;
      B.PixelFormat := pf24bit;
      MF := QRPrinter.GetPage(i);
      B.Width := MF.Width;
      B.Height := MF.Height;
      B.Canvas.Draw(0, 0, MF);
      B.PixelFormat := pf8bit;
      B.SaveToFile(FileName + PadL(IntToStr(i), 5) + '.BMP');
      B.Free;
      Prg.PB.StepIt;
    end;
  finally
    Prg.Free;
  end;
end;

procedure TExportQR.ExportQRJPG(FileName: string; FromPreview: boolean = False);
var
  i: integer;
  MF: TMetaFile;
  B: TBitmap;
  J: TJPEGImage;
  Prg: TFPrgrs;
  QRPrinter: TQRPrinter;
begin
  if (not FromPreview) then
    QRPrinter := FReport.QRPrinter
  else
    QRPrinter := FPreview.QRPrinter;
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := QRPrinter.PageCount - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(QRPrinter.PageCount);
    Prg.Accion.Update;
    Prg.Show;
    for i := 1 to QRPrinter.PageCount do
    begin
      Prg.Accion.Caption := Pagina + IntToStr(i) + de + IntToStr(QRPrinter.PageCount);
      Prg.Accion.Update;
      B := TBitmap.Create;
      J := TJPEGImage.Create;
      B.PixelFormat := pf24bit;
      MF := QRPrinter.GetPage(i);
      B.Width := MF.Width;
      B.Height := MF.Height;
      B.Canvas.Draw(0, 0, MF);
      B.PixelFormat := pf8bit;
      J.Assign(B);
      J.SaveToFile(FileName + PadL(IntToStr(i), 5) + '.JPG');
      B.Free;
      J.Free;
      Prg.PB.StepIt;
    end;
  finally
    Prg.Free;
  end;
end;

procedure TExportQR.ExportQREMF(FileName: string; FromPreview: boolean = False);
var
  i: integer;
  MF: TMetaFile;
  Prg: TFPrgrs;
  QRPrinter: TQRPrinter;
begin
  if (not FromPreview) then
    QRPrinter := FReport.QRPrinter
  else
    QRPrinter := FPreview.QRPrinter;
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := QRPrinter.PageCount - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(QRPrinter.PageCount);
    Prg.Accion.Update;
    Prg.Show;
    for i := 1 to QRPrinter.PageCount do
    begin
      Prg.Accion.Caption := Pagina + IntToStr(i) + De + IntToStr(QRPrinter.PageCount);
      Prg.Accion.Update;
      MF := QRPrinter.GetPage(i);
      MF.Enhanced := True;
      MF.SaveToFile(FileName + PadL(IntToStr(i), 5) + '.EMF');
      Prg.PB.StepIt;
    end;
  finally
    Prg.Free;
  end;
end;

procedure TExportQR.ExportQRWMF(FileName: string; FromPreview: boolean = False);
var
  i: integer;
  MF: TMetaFile;
  Prg: TFPrgrs;
  QRPrinter: TQRPrinter;
begin
  if (not FromPreview) then
    QRPrinter := FReport.QRPrinter
  else
    QRPrinter := FPreview.QRPrinter;
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := QRPrinter.PageCount - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(QRPrinter.PageCount);
    Prg.Accion.Update;
    Prg.Show;
    for i := 1 to QRPrinter.PageCount do
    begin
      Prg.Accion.Caption := Pagina + IntToStr(i) + De + IntToStr(QRPrinter.PageCount);
      Prg.Accion.Update;
      MF := QRPrinter.GetPage(i);
      MF.Enhanced := False;
      MF.SaveToFile(FileName + PadL(IntToStr(i), 5) + '.WMF');
      Prg.PB.StepIt;
    end;
  finally
    Prg.Free;
  end;
end;

procedure TExportQR.ExportQRPDF(FileName: string; Compresion: boolean; FromPreview: boolean = False);

  function CmsToPoints(Value: Double): Integer;
  begin
    Result := Round(Value * 0.01 * 0.393700787 * 72);
  end;

var
  PPDF: TPrintPDF;
  i: integer;
  MF: TMetaFile;
  B: TBitmap;
  Prg: TFPrgrs;
  QRPrinter: TQRPrinter;
  // 612 792
begin
  if (not FromPreview) then
    QRPrinter := FReport.QRPrinter
  else
    QRPrinter := FPreview.QRPrinter;
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := QRPrinter.PageCount - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(QRPrinter.PageCount);
    Prg.Accion.Update;
    Prg.Show;
    PPDF := TPrintPDF.Create(Self);
    PPDF.FileName := FileName + '.PDF';
    PPDF.Title := Title;
    PPDF.Author := Author;
    PPDF.Creator := Creator;
    PPDF.Keywords := Keywords;
    PPDF.Subject := Subject;
    PPDF.Producer := Producer;
    PPDF.BeginDoc;
    PPDF.PageWidth := CmsToPoints(QRPrinter.PaperWidth);//843;//792;
    PPDF.PageHeight := CmsToPoints(QRPrinter.PaperLength);//598;//612;
    {if FReport.Page.Orientation = poLandscape then
    begin
      PPDF.PageWidth := 843;//792;         mlfun
      PPDF.PageHeight := 598;//612;
    end;}
    for i := 1 to QRPrinter.PageCount do
    begin
      Prg.Accion.Caption := Pagina + IntToStr(i) + De + IntToStr(QRPrinter.PageCount);
      Prg.Accion.Update;
      B := TBitmap.Create;
      B.PixelFormat := pf24bit;
      MF := QRPrinter.GetPage(i);
      {B.Width := MF.Width;
      B.Height := MF.Height;
      B.Canvas.Draw(0, 0, MF);
      B.PixelFormat := pf8bit;}
      B.Width := MF.Width * 2;
      B.Height := MF.Height * 2;
      B.Canvas.Stretchdraw(Rect(0, 0, B.Width, B.Height), MF);
      if Compresion then
        PPDF.DrawBitmap(52, 0, B) // ZLIB
      else
        PPDF.DrawJPEG(52, 0, B); // JPEG
      if (i <> QRPrinter.PageCount) then
        PPDF.NewPage;
      Prg.PB.StepIt;
      B.Free;
    end;
  finally
    PPDF.EndDoc;
    PPDF.Free;
    Prg.Free;
  end;
end;

procedure TExportQR.ExportQRToStream(Stream: TStream; Compresion: Boolean = True);

  function CmsToPoints(Value: Double): Integer;
  begin
    Result := Round(Value * 0.01 * 0.393700787 * 72);
  end;

var
  PPDF: TPrintPDF;
  i: integer;
  MF: TMetaFile;
  B: TBitmap;
  Prg: TFPrgrs;
  QRPrinter: TQRPrinter;
  // 612 792
begin
  if Assigned(FReport) then
    QRPrinter := FReport.QRPrinter
  else
    QRPrinter := FPreview.QRPrinter;
  try
    // Vamos a guardar
    {Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := QRPrinter.PageCount - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(QRPrinter.PageCount);
    Prg.Accion.Update;
    Prg.Show;}
    PPDF := TPrintPDF.Create(Self);
    PPDF.Stream := Stream;
    PPDF.Title := Title;
    PPDF.Author := Author;
    PPDF.Creator := Creator;
    PPDF.Keywords := Keywords;
    PPDF.Subject := Subject;
    PPDF.Producer := Producer;
    PPDF.BeginDoc;
    PPDF.PageWidth := CmsToPoints(QRPrinter.PaperWidth);//843;//792;
    PPDF.PageHeight := CmsToPoints(QRPrinter.PaperLength);//598;//612;
    {if FReport.Page.Orientation = poLandscape then
    begin
      PPDF.PageWidth := 843;//792;         mlfun
      PPDF.PageHeight := 598;//612;
    end;}
    for i := 1 to QRPrinter.PageCount do
    begin
      //Prg.Accion.Caption := Pagina + IntToStr(i) + De + IntToStr(QRPrinter.PageCount);
      //Prg.Accion.Update;
      B := TBitmap.Create;
      B.PixelFormat := pf24bit;
      MF := QRPrinter.GetPage(i);
      {B.Width := MF.Width;
      B.Height := MF.Height;
      B.Canvas.Draw(0, 0, MF);
      B.PixelFormat := pf8bit;}
      B.Width := MF.Width * 2;
      B.Height := MF.Height * 2;
      B.Canvas.Stretchdraw(Rect(0, 0, B.Width, B.Height), MF);
      if Compresion then
        PPDF.DrawBitmap(52, 0, B) // ZLIB
      else
        PPDF.DrawJPEG(52, 0, B); // JPEG
      if (i <> QRPrinter.PageCount) then
        PPDF.NewPage;
      Prg.PB.StepIt;
      B.Free;
    end;
  finally
    PPDF.EndDoc;
    PPDF.Free;
    //Prg.Free;
  end;
end;
{$ENDIF}

(*****************************)
(*        TEXPORTFR          *)
(*****************************)
{$IFDEF FASTREPORT}

constructor TExportFR.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

destructor TExportFR.Destroy;
begin
  inherited Destroy;
end;

procedure TExportFR.ExportFR;
var
  SD: TSaveDialog;
  Ruta: string;
  Fich: string;
  P: integer;
begin
  if (not Assigned(FReport)) then
    Exit;
  FReport.PrepareReport; {Prepare report}
  try
    // Sacamos el cuadro de dilogo para guardar
    SD := TSaveDialog.Create(Self);
    SD.Title := Guardar;
    SD.Filter := Filtro;
    SD.FilterIndex := 0;
    if (SD.Execute) then
    begin
      Ruta := ExtractFilePath(SD.FileName);
      Fich := ExtractFileName(SD.FileName);
      P := Pos('.', Fich);
      if (P <> 0) then
        Fich := Copy(Fich, 1, P - 1);
      case SD.FilterIndex of
        1:
          begin
            ExportFRPDF(Ruta + Fich, True);
          end;
        2:
          begin
            ExportFRPDF(Ruta + Fich, False);
          end;
        3:
          begin
            ExportFRJPG(Ruta + Fich);
          end;
        4:
          begin
            ExportFRBMP(Ruta + Fich);
          end;
        5:
          begin
            ExportFREMF(Ruta + Fich);
          end;
        6:
          begin
            ExportFRWMF(Ruta + Fich);
          end;
      end;
    end;
    SD.Free;
  except
    MessageDlg(Error, mtError, [mbOK], 0);
    Exit;
  end;
end;

procedure TExportFR.ExportFRBMP(FileName: string);
var
  i: integer;
  B: TBitmap;
  Prg: TFPrgrs;
  Ancho, Alto: Longint;
begin
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := FReport.EMFPages.Count - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(FReport.EMFPages.Count);
    Prg.Accion.Update;
    Prg.Show;
    for i := 0 to FReport.EMFPages.Count - 1 do
    begin
      Ancho := FReport.EMFPages[i].PrnInfo.Pgw;
      Alto := FReport.EMFPages[i].PrnInfo.Pgh;
      Prg.Accion.Caption := Pagina + IntToStr(i + 1) + De + IntToStr(FReport.EMFPages.Count);
      Prg.Accion.Update;
      B := TBitmap.Create;
      B.PixelFormat := pf24bit;
      B.Width := Ancho;
      B.Height := Alto;
      FReport.EMFPages.Pages[i].Visible := True;
      FReport.EMFPages.Draw(i, B.Canvas, Rect(0, 0, Ancho, Alto));
      B.PixelFormat := pf8bit;
      B.SaveToFile(FileName + PadL(IntToStr(i + 1), 5) + '.BMP');
      B.Free;
      Prg.PB.StepIt;
    end;
    Prg.Free;
  except
    MessageDlg(Error, mtError, [mbOK], 0);
    Exit;
  end;
end;

procedure TExportFR.ExportFRJPG(FileName: string);
var
  i: integer;
  B: TBitmap;
  J: TJPEGImage;
  Prg: TFPrgrs;
  Ancho, Alto: Longint;
begin
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := FReport.EMFPages.Count - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(FReport.EMFPages.Count);
    Prg.Accion.Update;
    Prg.Show;
    for i := 0 to FReport.EMFPages.Count - 1 do
    begin
      Ancho := FReport.EMFPages[i].PrnInfo.Pgw;
      Alto := FReport.EMFPages[i].PrnInfo.Pgh;

      Prg.Accion.Caption := Pagina + IntToStr(i + 1) + De + IntToStr(FReport.EMFPages.Count);
      Prg.Accion.Update;
      B := TBitmap.Create;
      J := TJPEGImage.Create;
      B.PixelFormat := pf24bit;
      B.Width := Ancho;
      B.Height := Alto;
      FReport.EMFPages.Pages[i].Visible := True;
      FReport.EMFPages.Draw(i, B.Canvas, Rect(0, 0, Ancho, Alto));
      B.PixelFormat := pf8bit;
      J.Assign(B);
      J.SaveToFile(FileName + PadL(IntToStr(i + 1), 5) + '.JPG');
      B.Free;
      J.Free;
      Prg.PB.StepIt;
    end;
    Prg.Free;
  except
    MessageDlg(Error, mtError, [mbOK], 0);
    Exit;
  end;
end;

procedure TExportFR.ExportFREMF(FileName: string);
var
  i: integer;
  MF: TMetaFile;
  MFC: TMetafileCanvas;
  Prg: TFPrgrs;
  Ancho, Alto: Longint;
begin
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := FReport.EMFPages.Count;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(FReport.EMFPages.Count);
    Prg.Accion.Update;
    Prg.Show;
    for i := 0 to FReport.EMFPages.Count - 1 do
    begin
      Ancho := FReport.EMFPages[i].PrnInfo.Pgw;
      Alto := FReport.EMFPages[i].PrnInfo.Pgh;
      MF := TMetafile.Create;
      MF.Width := Ancho;
      MF.Height := Alto;
      MFC := TMetafileCanvas.Create(MF, 0);
      Prg.Accion.Caption := Pagina + IntToStr(i + 1) + De + IntToStr(FReport.EMFPages.Count);
      Prg.Accion.Update;
      FReport.EMFPages.Pages[i].Visible := True;
      FReport.EMFPages.Draw(i, MFC, Rect(0, 0, Ancho, Alto));
      MFC.Free;
      MF.Enhanced := True;
      MF.SaveToFile(FileName + Padl(IntToStr(I + 1), 5) + '.EMF');
      MF.Free;
      Prg.PB.StepIt;
    end;
    Prg.Free;
  except
    MessageDlg(Error, mtError, [mbOK], 0);
    Exit;
  end;
end;

procedure TExportFR.ExportFRWMF(FileName: string);
var
  i: integer;
  MF: TMetaFile;
  MFC: TMetafileCanvas;
  Prg: TFPrgrs;
  Ancho, Alto: Longint;
begin
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := FReport.EMFPages.Count - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(FReport.EMFPages.Count);
    Prg.Accion.Update;
    Prg.Show;
    for i := 0 to FReport.EMFPages.Count - 1 do
    begin
      Ancho := FReport.EMFPages[i].PrnInfo.Pgw;
      Alto := FReport.EMFPages[i].PrnInfo.Pgh;
      MF := TMetafile.Create;
      MF.Width := Ancho;
      MF.Height := Alto;
      MFC := TMetafileCanvas.Create(MF, 0);
      Prg.Accion.Caption := Pagina + IntToStr(i + 1) + De + IntToStr(FReport.EMFPages.Count);
      Prg.Accion.Update;
      FReport.EMFPages.Pages[i].Visible := True;
      FReport.EMFPages.Draw(i, MFC, Rect(0, 0, Ancho, Alto));
      MFC.Free;
      MF.Enhanced := False;
      MF.SaveToFile(FileName + Padl(IntToStr(I + 1), 5) + '.WMF');
      MF.Free;
      Prg.PB.StepIt;
    end;
    Prg.Free;
  except
    MessageDlg(Error, mtError, [mbOK], 0);
    Exit;
  end;
end;

procedure TExportFR.ExportFRPDF(FileName: string; Compresion: boolean);
var
  PPDF: TPrintPDF;
  i: integer;
  B: TBitmap;
  Prg: TFPrgrs;
  Ancho, Alto: Longint;
begin
  try
    // Vamos a guardar
    Prg := TFPrgrs.Create(Self);
    Prg.Caption := Exportando;
    Prg.PB.Min := 0;
    Prg.PB.Max := FReport.EMFPages.Count - 1;
    Prg.PB.Position := 0;
    Prg.PB.Update;
    Prg.Accion.Caption := Pagina1De + IntToStr(FReport.EMFPages.Count);
    Prg.Accion.Update;
    Prg.Show;
    PPDF := TPrintPDF.Create(Self);
    PPDF.FileName := FileName + '.PDF';
    PPDF.BeginDoc;
    for i := 0 to FReport.EMFPages.Count - 1 do
    begin
      Ancho := FReport.EMFPages[i].PrnInfo.Pgw;
      Alto := FReport.EMFPages[i].PrnInfo.Pgh;
      if (i = 0) and (Ancho > Alto) then
      begin
        PPDF.PageWidth := 792;
        PPDF.PageHeight := 612;
      end;
      Prg.Accion.Caption := Pagina + IntToStr(i + 1) + De + IntToStr(FReport.EMFPages.Count);
      Prg.Accion.Update;
      B := TBitmap.Create;
      B.PixelFormat := pf24bit;
      B.Width := Ancho;
      B.Height := Alto;
      FReport.EMFPages.Pages[i].Visible := True;
      FReport.EMFPages.Draw(i, B.Canvas, Rect(0, 0, Ancho, Alto));
      B.PixelFormat := pf8bit;
      if (Compresion) then
        PPDF.DrawBitmap(52, 0, B) // ZLIB
      else
        PPDF.DrawJPEG(52, 0, B); // JPEG
      if (i <> FReport.EMFPages.Count - 1) then
        PPDF.NewPage;
      Prg.PB.StepIt;
      B.Free;
    end;
    PPDF.EndDoc;
    PPDF.Free;
    Prg.Free;
  except
    MessageDlg(Error, mtError, [mbOK], 0);
    Exit;
  end;
end;
{$ENDIF}
end.

